﻿/**
///////////////////////////////////////////////////////////////////////////////
ADOBE SYSTEMS INCORPORATED
Copyright 2008 Adobe Systems Incorporated
All Rights Reserved.
NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
terms of the Adobe license agreement accompanying it.  If you have received this file from a 
source other than Adobe, then your use, modification, or distribution of it requires the prior 
written permission of Adobe.
///////////////////////////////////////////////////////////////////////////////

DESCRIPTION

This sample shows how to create a user interface for your Adobe Device Central project
plug-in.
1. It builds a complex configuration window using most of the user interface elements 
   available in the task plug-in XML file. 
2. Pre-populates some of the dialog elements from the plug-in script file.
3. Reads and validates the user input so it can be used when processing the task. 

For more information see "Creating a User Interface for Your Task" in the 
Adobe Device Central CS4 SDK Programmer’s Guide.

This sample also demonstrates:
1. How to create an archive file using the Adobe Device Central ZIPFile class.
2. How to access device profile information using Adobe Device Central Device
    class and the ExtendScript XML feature.

For more information on the ZIPFile and Device objects, see the Adobe Device Central 
CS4 SDK Reference Guide.

For more information on processing XML with ExtendScript, see the "Integrating XML 
into JavaScript" section in the JavaScript Tools Guide CS4 that is part of this SDK. This guide is 
also available in the ExtendScript Toolkit application under Help > SDK.

HOW TO USE

1. Copy the sample plug-in to the <TASKS> folder as specified in the Adobe Device Central CS4 
    Programmer's Guide.
2. Start Device Central and create a new project. The "Create Zip" toolbar icon and menu 
    option should now be available.
3. Add some resources to the project.
4. Add some devices to the project.
5. Select some of the resources and click on the "Create Zip" icon in the toolbar.
6. The task's configuration dialog will be displayed pre-populated with the 
    selected resources and the available device profiles.
7. Enter the archive name and select the "zip" option from the "Extension" field.
8. Click on the "Browse" button to launch a folder browser and select the destination
    where the archive should be saved.
9. Uncheck the "Save full path info" to create a flat archive file.
10. Select the device profiles to be added to the archive.
11. Finally, click "Run" to create a zip file that will contain the resources in the list
     and the device profiles selected.
12. If the task runs successfully, a new zip file will be created in the folder specified
     with the name indicated.

*/


/** Constants for the different types of extensions used in this sample to populate the 
extensions comboBox */
var EXTENSION_CUSTOM = "<custom>";
var EXTENSION_ZIP = "zip";

/** Global variable that represents the zip file created by the task. Needs to be global so it
can be access by the different functions*/
var gZip = null;

// ------------------------------------------
// Helper methods
// ------------------------------------------

/** Updates the status dialog to complete with the message passed */
function doneStatusDialog(message)
{
	var dialog = task.statusDialog;
	dialog.text = message;
	dialog.enableDoneButton(true);
	dialog.enableCancelButton(false);
}

/** Creates an array of ListItem objects necessary to populate the devices listBox */
function getDevices(selectedDevices) 
{
	//Retrieve all the devices available in the project
	var devices = project.devices;
	var list =[];
	for(var i = 0; i < devices.length; i++)
	{
		/*Create a ListItem object for each device to display with the following properties:
		model, make and path*/
		var device = devices[i];
		
		//Build an XML object with the contents of the profileXML string 
		var profileXML = new XML(device.profileXML);
		
		var file = new File(device.profilePath);
		
		var item = new ListItem;
		item.model = device.name;
		//Retrieve the make information from the device XML profile using ExtendScript XML parsing
		item.make = profileXML.coreData.manufacturer[0].@value
		item.path = file.fsName;
		
		/* If there are selected devices from the previous run of that task, check if the item created
		was selected and update the selected attribute accordingly. If selected is "true" the device
		will show in the dialog with a ticked checkbox */
		if (selectedDevices)
			item.selected = inArray(selectedDevices, item);
			
		list.push(item);
	}
	return list;
}

/** Builds an array of ListItems to populate the extensions comboBox*/
function getExtensions()
{
	var list = [];
	var extensions = [EXTENSION_ZIP, EXTENSION_CUSTOM];
	for(var i = 0; i < extensions.length; i++)
	{	
		var item = new ListItem;
		item.value = extensions[i];
		list.push(item);
	}
	return list;
}

/** Builds an array of ListItems necessary to populate the fileList in the configuration dialog*/
function getSelectedResources() 
{
	var selectedFiles = project.selectedResources;
	var resources =[];
	for(var i = 0; i < selectedFiles.length; i++)
	{
		var file = new File(selectedFiles[i]);
		
		var item = new ListItem;
		item.filename = file.displayName;
		item.path = file.fsName;
		resources.push(item);
	}
	return resources;
}

/** Verifies if the element passed is part of a given list of items */
function inArray(list, elem)
{
	for(var i = 0; (i < list.length) ; i++)
		if(elem.path == list[i].path) return true;
	
	return false;
}

/** Sets up and launches the status dialog */
function launchStatusDialog()
{
	var dialog = task.statusDialog;
	dialog.start("doCancel", "doDone");
	dialog.title = localize("$$$/Preferences/Processing=Processing Create Zip task...");
	dialog.text = localize("$$$/Preferences/Archiving=Archiving files");
	dialog.enableDoneButton(false);
	dialog.enableCancelButton(true);
}

function toArray(o)
{
	var a = [];
	for(i = 0; i < o.length; i++)
		a.push(o[i]);
	
	return a;
}

// ------------------------------------------
// Methods invoked from the UI
// ------------------------------------------

/** Invokes a resources dialog and adds the files selected in the resources dialog 
to the files fileList control in the configuration dialog. */
function doAddFile()
{
	var newFiles = project.openResourcesDialog ();
	
	if(newFiles != undefined)
	{
		var files = toArray(task.getFileList("files").list);
		
		for(var i = 0; i < newFiles.length; i++)
		{
			var f = new File(newFiles[i]);
			var item = new ListItem;
			item.filename = f.displayName;
			item.path = f.fsName;
			
			if(! inArray(files, item))
				files.push(item);
		}
		
		task.getFileList("files").list = files;
		
		if(files.length > 0)
			task.getButton("btnRemove").enabled = true;
	}
}

/**If the task is canceled during execution, closes the ZIPFile object if already initialized and
closes the status dialog*/
function doCancel()
{
	if (gZip != undefined)
		gZip.close();
		
	task.statusDialog.end();
}

/** Resets the list of devices so none is selected */
function doClear()
{
	task.getListBox("devices").list = getDevices();	
}

/** Closes the status dialog when the task has completed*/
function doDone()
{
	task.statusDialog.end();
}

/** Removes the files selected from the fileList in the configuration dialog */
function doRemoveFile()
{
	var filesControl = task.getFileList("files");
	var selectedFiles = filesControl.selectedItems;
	var all = filesControl.list;
	var tmpFiles = [];
	for(i = 0; i < all.length; i++)
	{
			var found = false;
			for(j = 0; (j < selectedFiles.length) && ! found; j++)
			{
				if(all[i].path == selectedFiles[j].path)
					found = true;
			}
			if(! found)
				tmpFiles.push(all[i]);
	}
	if(tmpFiles.length <= 0)
		task.getButton("btnRemove").enabled = false;
		
	filesControl.list = tmpFiles;
}

/** Opens an OS specific folder browser and updates the folder textField with the 
location of the folder selected by the user */
function openDialog()
{
	var selectedFolder = Folder.selectDialog();
	if (selectedFolder != undefined)
	{
		task.getTextField("folder").text = selectedFolder.fsName;
	}
}

/** If custom extension has been selected enables the custom extension text field. 
Otherwise, disable it.*/
function toggleCustomExtension()
{
	var customExtensionField = task.getTextField("customExtension");
	if(EXTENSION_CUSTOM == task.getComboBox("extension").selectedItem.value)
	{
		customExtensionField.enabled = true;
	}
	else
	{
		customExtensionField.enabled = false;
		//Re-set the custom extension field if no longer selected
		customExtensionField.text = "";
	}
}

// ------------------------------------------
// install
// ------------------------------------------

/** On installation disables the remove and clear buttons since no files or devices have been loaded yet. 
Pre-populates the extension comboBox */
function install()
{
	task.getComboBox("extension").list = getExtensions();
	task.getTextField("customExtension").enabled = false;
	
	/* When installing the task there's no files added to the files list or devices to the devices list. 
	We won't be able to remove or uncheck any so we need to disable  "btnRemove" and "btnClear" */	
	task.getButton("btnRemove").enabled = false;
	task.getButton("btnClear").enabled = false;
}

// ------------------------------------------
// configure
// ------------------------------------------

/** Sets up the configuration dialog with the extensions, devices and files available.
If the task has been invoked from the toolbar, makes sure the extension comboBox and the files
fileList are populated */
function configure(toolbar)
{ 
	if (toolbar)
	{
		/*Toolbar install mechanism doesn't remember UI status so the extensions comboBox must
		be populated here */
		var extensionCache = task.getComboBox("extension").selectedItem;
		if (task.getComboBox("extension").list.length == 0);
			task.getComboBox("extension").list = getExtensions();
		if (extensionCache) {task.getComboBox("extension").selectedItem = extensionCache;} 
		
		/* If we have selected to run the task from the toolbar we need to add the 
		pre selected files to the fileList in the configuration dialog. */
		task.getFileList("files").list = getSelectedResources();
		
		/* The remove buttons should now be enabled since there are files 
		that can be removed from the list. */
		task.getButton("btnRemove").enabled = true;
	}

	//load all the project devices with the already selected ones ticked
	var selectedDevices = task.getListBox("devices").selectedItems;
	task.getListBox("devices").list = getDevices(selectedDevices);	
	
	/* The clear button should now be enabled since could be devices
	that can be unselected. */
	task.getButton("btnClear").enabled = true;
	
	//Make sure the custom extension element is properly enabled/disabled
	toggleCustomExtension();
}

// ------------------------------------------
// execute
// ------------------------------------------

/** Creates a zip file based on the user preferences */
function execute()
{
	launchStatusDialog();
	
	//Retrieve the user input
	var devices = task.getListBox("devices").selectedItems;
	var resources = task.getFileList("files").list;
	var filename = task.getTextField("name").text;
	var extension = task.getComboBox("extension").selectedItem.value;
	var folder = task.getTextField("folder").text;
	var fullPath = task.getCheckBox("fullpath").checked;
	
	if (extension == EXTENSION_CUSTOM)
	{
		extension = task.getTextField("customExtension").text;
	}
	
	//Validate the user input
	if (filename == undefined || filename.length <= 0) 
	{
		doneStatusDialog(localize("$$$/Preferences/NoName=No archive name has been provided."));
	}
	else if (extension <= 0)
	{
		doneStatusDialog(localize("$$$/Preferences/NoExtension=Please select an extension from the list or type your custom extension."));
	}
	else if (folder == undefined || folder.length <= 0) 
	{
		doneStatusDialog(localize("$$$/Preferences/NoFolder=No storage folder has been provided."));
	}
	else if (! new Folder(folder).exists)
	{
		doneStatusDialog(localize("$$$/Preferences/NotExistingFolder=The folder provided does not exists."));
	}
	else if (devices.length <= 0 && resources.length <= 0)
	{
		doneStatusDialog(localize("$$$/Preferences/NoFiles=No devices or resources have been selected."));
	}
	else
	{
		//Create the archive file and open it passing the required path, name and extension
		gZip = new ZIPFile;
		var zipPath = folder + (task.isOSMacintosh ? "/" : "\\") +  filename + "." + extension;
		gZip.open(zipPath);
		
		//Pre-define a date stamped message to add to each file as a comment
		var now = new Date().toString();
		var comment = localize("$$$/Preferences/ZipComment=Added by ADC CS4 on %1.", now);
		
		//Add to the zip file the profile XML files of the devices selected
		for (var i = 0; i < devices.length; i++)
		{
			 var profile = devices[i].path;
			 var name = fullPath ? profile : new File(profile).displayName;
			 gZip.addFile(name, profile, comment );
		}

		//Add all the resources selected to the archive
		for (var i = 0; i < resources.length; i++)
		{
			var resource = resources[i].path;
			var name = fullPath ? resource : new File(resource).displayName;
			gZip.addFile(name, resource, comment );
		}
		
		//Close the file once all the files have been added
		gZip.close(); 
		
		var zipFile = new File(zipPath);
		
		//Verify that the file does exist and therefore has been created successfully
		if (zipFile.exists)
			doneStatusDialog(localize("$$$/Preferences/ZipPath=The archive %1 has been successfully created.", zipFile.fsName));
		else
			doneStatusDialog(localize("$$$/Preferences/NoZipCreated=The archive could not been created."));
  }
}
