﻿/**
///////////////////////////////////////////////////////////////////////////////
ADOBE SYSTEMS INCORPORATED
Copyright 2008 Adobe Systems Incorporated
All Rights Reserved.
NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
terms of the Adobe license agreement accompanying it.  If you have received this file from a 
source other than Adobe, then your use, modification, or distribution of it requires the prior 
written permission of Adobe.
///////////////////////////////////////////////////////////////////////////////

DESCRIPTION

This sample shows how to add metadata to SWF resources from within your Device Central project 
using Adobe's Extensible Metadata Platform (XMP). To read and write the information, the script 
uses XMPScript, the XMP ExtendScript API, that offers JavaScript access to the Adobe XMP Core 
and XMPFiles libraries. This sample follows the rules for embedding XMP in SWF files as specified in 
the SWF file format specification. 

For more information about XMP metadata see http://www.adobe.com/products/xmp.

For more information about XMP ExtendScript API see the "Scripting Access to XMP Metadata" 
section in the JavaScript Tools Guide CS4. This guide is available with this SDK or in the ExtendScript 
Toolkit application under Help > SDK.

Adobe Device Central doesn't provide the XMP libraries; therefore, they must be added to your application 
library folder and loaded as an ExtendScript ExternalObject. The following three libraries are required 
and are provided with this task in DLL (for Windows) and Framework (for Mac OS) formats:

1. AdobeXMP
2. AdobeXMPFiles
3. AdobeXMPScript

In Windows, copy AdobeXMP.dll, AdobeXMPFiles.dll and AdobeXMPScript.dll from the "libraries/win"
folder within the task to the Device Central CS4 installation folder:

"C:\Program Files\Adobe\Adobe Device Central CS4".

In Mac OS, copy AdobeXMP.framework, AdobeXMPFiles.framework and AdobeXMPScript.framework
from the "libraries/mac" folder within the task to the application Frameworks folder:

"Applications/Adobe Device Central CS4/Device Central.app/Contents/Frameworks/".

The following are the XMP properties included in the sample:

1. Title: title of the content.
2. Author: name of the content author.
3. Description: description of the content.
4. Copyright information: relevant copyright details.
5. Copyright URL: online address of the copyright information
6. Content type: content category.
7. Content version: version number of the content (not Flash version).
8. Storage size: size of SharedObject required by the content.
9. Background alpha: blend value for the background transparency feature.
10: Forward lock: specifies if the content file can be transferred to another device.

HOW TO USE

1. Copy the sample plug-in to the <TASKS> folder as specified in the Adobe Device Central CS4
    Programmer's Guide.
2. Make sure the XMP libraries are copied to the library directory within Adobe Device Central 
    as explained above.
3. Start Device Central and create a new project. The "Edit SWF Metadata" toolbar icon and menu 
    option should now be available.
4. Add a SWF resource to the project.
5. Select the SWF file and click on the "Edit SWF Metadata" icon in the toolbar.
6. The name of the file should show at the top of the configuration dialog. If the SWF file already had 
    XMP metadata available the corresponding fields will be populated in the "Metadata to edit" and
    "Current metadata" columns. If no metadata is already stored in the file the fields will be blank.
7. Add or update the metadata by filling in the fields in the "Metadata to edit" column of the configuration dialog.
8. Click "Run" to update the file.
9. If the file accepts XMP metadata a success message will be displayed.
10. You can use Adobe Bridge to verify that the data has been correctly updated. Find the file updated in 
    Adobe Bridge, right click on the file and select "File Info..." from the context menu. The new XMP 
    data should show under the tab "Mobile SWF" and "Raw Data".
*/

/** List of possible SWF content types */
const CONTENT_TYPES = ["application", "callingscreen", "channel", "game", 
	"homescreen", "icon", "incomingcall", "movie", "other", "presentation", 
	"receivingmail", "ringtone", "screensaver", "sendingmail", "shutdown", 
	"skin", "startup", "video", "wakeupscreen", "wallpaper"];
	
//Load the XMP library
if (ExternalObject.AdobeXMPScript == undefined) 
	ExternalObject.AdobeXMPScript = new ExternalObject('lib:AdobeXMPScript');
	
// ------------------------------------------
// Helper methods
// ------------------------------------------

/** Builds an array of ListItem objects necessary to populate the contentType comboBox control. 
    The selected argument indicates the value that must be pre-selected in the comboBox.
*/
function getContentTypesList(selected)
{
	var types = [];

	for(var i = 0; i < CONTENT_TYPES.length; i++)
	{
		var item = new ListItem;
		item.value = CONTENT_TYPES[i];

		if (item.value == selected)
			item.selected = true;
		types.push(item);
	}		

	return types;
}

/** Validates if the file path passed corresponds to a SWF file by checking the extension of the file.
Returns true if the file is a SWF, false otherwise.*/
function isFileSWF(filePath)
{
	var parts = filePath.split('.');
	var extension = parts.length > 1 ? parts.pop() : '';
	return extension.toLowerCase() == "swf";
}

/** Sets up and launches the status dialog */
function launchStatusDialog()
{
	var dialog = task.statusDialog;
	dialog.start("doCancel", "doDone");
	dialog.title = localize("$$$/Metadata/Edit=Edit SWF Metadata");
	dialog.text = localize("$$$/Metadata/ProcessingRequest=Updating file...");
	dialog.enableDoneButton(false);
	dialog.enableCancelButton(true);
}

/** Sets the value of an array metadata property. Deletes the existing array for the property and 
creates a new one with the value passed.*/
function setArray(xmp, schema, property, value, flags)
{
	xmp.deleteProperty(schema, property);
	xmp.appendArrayItem(schema, property, value, 0, flags);
}

/** Sets the value of an alternate-text array metadata property. Deletes the existing contents before
setting the property to the value passed.*/
function setLocalizedText(xmp, schema, property, value)
{
	xmp.deleteProperty(schema, property);
	xmp.setLocalizedText(schema, property, null, "x-default", value);
}

/** Sets the value of a simple metadata property. If no value has been passed, deletes the existing value */
function setProperty(xmp, schema, property, value)
{
	if (value == null || value.length == 0)
		xmp.deleteProperty(schema, property);
	else
		xmp.setProperty(schema, property, value);
}

/* Prepares the dialog by disabling the fields in the "Current metadata" column so
they cannot be edited */
function prepareDialog()
{
	task.getTextField("curtitle").enabled = false;
	task.getTextField("curauthor").enabled  = false;
	task.getTextField("curdescription").enabled = false;
	task.getTextField("curcopyright").enabled = false;
	task.getTextField("curcopyrightURL").enabled = false;
	task.getTextField("curcontentType").enabled = false;
	task.getTextField("curcontentVersion").enabled = false;
	task.getTextField("curstorage").enabled = false;
	task.getTextField("curalpha").enabled = false;
	task.getCheckBox("curlock").enabled = false;
}

/** Udates the fields in the configuration dialog with XMP data already stored in the file passed.*/
function populateDialog(filePath, duplicate)
{			
	prepareDialog();

	//Register the SWF namespace if not already available
	if (XMPMeta.getNamespaceURI("swf").length == 0)
			XMPMeta.registerNamespace(XMPConst.NS_SWF, "swf");

	//Create a XMPFile object and retrieve the XMP information
	var xmpFile = new XMPFile( filePath, "FILE_SWF", XMPConst.OPEN_FOR_READ);
	var xmp = xmpFile.getXMP();

	//Set the value of the resource control with the path of the file selected
	task.getStaticText("resource").label = filePath;
	
	//Set the controls with the existing XMP properties in the file. Otherwise, reset the control.
	task.getTextField("curtitle").text = xmp.getLocalizedText(XMPConst.NS_DC, "title", null, "x-default") || "";
	task.getTextField("curauthor").text  = xmp.getArrayItem(XMPConst.NS_DC, "creator", 1) || "";
	task.getTextField("curdescription").text = xmp.getLocalizedText(XMPConst.NS_DC, "description", null, "x-default") || "";
	task.getTextField("curcopyright").text = xmp.getLocalizedText(XMPConst.NS_DC, "rights", null, "x-default") || "";
	task.getTextField("curcopyrightURL").text = xmp.getProperty(XMPConst.NS_XMP_RIGHTS, "WebStatement") || "";
	task.getTextField("curcontentType").text = xmp.getProperty(XMPConst.NS_SWF, "type") || "";
	task.getTextField("curcontentVersion").text = xmp.getProperty(XMPConst.NS_XMP_MM, "VersionID") || "";
	task.getTextField("curstorage").text = xmp.getProperty(XMPConst.NS_SWF, "maxstorage") || "";
	task.getTextField("curalpha").text = xmp.getProperty(XMPConst.NS_SWF, "bgalpha") || "";
	task.getCheckBox("curlock").checked = xmp.getProperty(XMPConst.NS_SWF, "forwardlock");
	
	if(duplicate)
	{
		task.getTextField("title").text = xmp.getLocalizedText(XMPConst.NS_DC, "title", null, "x-default") || "";
		task.getTextField("author").text  = xmp.getArrayItem(XMPConst.NS_DC, "creator", 1) || "";
		task.getTextField("description").text = xmp.getLocalizedText(XMPConst.NS_DC, "description", null, "x-default") || "";
		task.getTextField("copyright").text = xmp.getLocalizedText(XMPConst.NS_DC, "rights", null, "x-default") || "";
		task.getTextField("copyrightURL").text = xmp.getProperty(XMPConst.NS_XMP_RIGHTS, "WebStatement") || "";
		task.getComboBox("contentType").list = getContentTypesList(xmp.getProperty(XMPConst.NS_SWF, "type"));
		task.getTextField("contentVersion").text = xmp.getProperty(XMPConst.NS_XMP_MM, "VersionID") || "";
		task.getTextField("storage").text = xmp.getProperty(XMPConst.NS_SWF, "maxstorage") || "";
		task.getTextField("alpha").text = xmp.getProperty(XMPConst.NS_SWF, "bgalpha") || "";
		task.getCheckBox("lock").checked = xmp.getProperty(XMPConst.NS_SWF, "forwardlock");
	}
}

/** Resets the value of all the controls in the configuration dialog */
function resetDialog()
{
	task.getStaticText("resource").label = "";
	
	//Reset the "Metadata to edit" section
	task.getTextField("title").text = "";
	task.getTextField("author").text  = "";
	task.getTextField("description").text = "";
	task.getTextField("copyright").text = "";
	task.getTextField("copyrightURL").text = "";
	task.getComboBox("contentType").list = getContentTypesList("application");
	task.getTextField("contentVersion").text = "";
	task.getTextField("storage").text = "";
	task.getTextField("alpha").text = "";
	task.getCheckBox("lock").checked = false;
	
	//Reset the "Current Metadata" section
	task.getTextField("curtitle").text = "";
	task.getTextField("curauthor").text  = "";
	task.getTextField("curdescription").text = "";
	task.getTextField("curcopyright").text = "";
	task.getTextField("curcopyrightURL").text = "";
	task.getTextField("curcontentType").text = "";
	task.getTextField("curcontentVersion").text = "";
	task.getTextField("curstorage").text = "";
	task.getTextField("curalpha").text = "";
	task.getCheckBox("curlock").checked = false;
}

// ------------------------------------------
// Methods invoked from the UI
// ------------------------------------------

/**If the task is canceled during execution, closes the status dialog */
function doCancel()
{
	task.statusDialog.end();
}

/** When the task is completed closes the status dialog.*/
function doDone()
{
	task.statusDialog.end();
}

/** Opens an OS specific file dialog and filters the files to only display SWFs. 
The file selected by the user is returned and if it exists updates the configuration dialog with
the file's XMP details. */
function getFile()
{
	var selectedFile = File.openDialog ("Select a file", "*.swf");
	if (selectedFile != null && isFileSWF(selectedFile.fsName))
		populateDialog(selectedFile.fsName, true);
	else
		resetDialog();
}

// ------------------------------------------
// install
// ------------------------------------------

function install()
{
	prepareDialog();
	task.getComboBox("contentType").list = getContentTypesList("application");
	task.getCheckBox("curlock").checked = false;
	task.getCheckBox("lock").checked = false;
}

// ------------------------------------------
// configure
// ------------------------------------------

/** Sets up the configuration dialog based on the SWF file selected. */
function configure(toolbar)
{
	prepareDialog();
	
	var selectedFile = "";
	
	if (toolbar)
	{
		 selectedFile = project.selectedResources[0];
		 task.getComboBox("contentType").list = getContentTypesList("application");
		 task.getCheckBox("lock").checked = false;
	}
	else
	{
		selectedFile = task.getStaticText("resource").label;
	}

	if (isFileSWF(selectedFile))
	{
		populateDialog(selectedFile, toolbar ? true : false);
	}
	else
	{
		alert(localize("$$$/Metadata/NotSWF=The resource file must be a SWF"));
		resetDialog();
	}
}

// ------------------------------------------
// execute
// ------------------------------------------

/** Updates the file XMP metadata with the data input by the user in the "Metadata to edit" column 
of the configuration dialog.*/
function execute()
{
	launchStatusDialog();

	//Retrieve the user input
	var title = task.getTextField("title").text;
	var author = task.getTextField("author").text;
	var description = task.getTextField("description").text;
	var copyright = task.getTextField("copyright").text;
	var copyrightURL = task.getTextField("copyrightURL").text;
	var contentType = task.getComboBox("contentType").selectedItem;
	var contentVersion = task.getTextField("contentVersion").text;
	var storage = task.getTextField("storage").text;
	var alpha = task.getTextField("alpha").text;
	var lock = task.getCheckBox("lock").checked;

	var filePath = task.getStaticText("resource").label;
	var file = new File(filePath);

	//Make sure the file selected is a SWF
	if (file.exists && isFileSWF(filePath) && ! file.readonly)
	{	
		//Register the SWF namespace if not already available
		if (XMPMeta.getNamespaceURI("swf").length == 0)
			XMPMeta.registerNamespace(XMPConst.NS_SWF, "swf");	
	
		//Create a  SWF XMPFile  object ,open it for update and retrieve the metadata.
		var xmpFile = new XMPFile( filePath, "FILE_SWF", XMPConst.OPEN_FOR_UPDATE);
		xmp = xmpFile.getXMP();

		//Update document title
		setLocalizedText(xmp, XMPConst.NS_DC, "title", title)

		//Update author
		setArray(xmp, XMPConst.NS_DC, "creator", author, XMPConst.PROP_IS_ARRAY | XMPConst.ARRAY_IS_ORDERED);

		//Update description 
		setLocalizedText(xmp, XMPConst.NS_DC, "description", description)

		//Update copyright notice
		setLocalizedText(xmp, XMPConst.NS_DC, "rights", copyright)

		//Update copyright URL
		setProperty(xmp, XMPConst.NS_XMP_RIGHTS, "WebStatement", copyrightURL);

		//Update content type
		setProperty(xmp, XMPConst.NS_SWF, "type", contentType.value);

		//Update content version
		setProperty(xmp, XMPConst.NS_XMP_MM, "VersionID", contentVersion);

		//Update persistent storage
		setProperty(xmp, XMPConst.NS_SWF, "maxstorage", storage);

		//Update background alpha
		setProperty(xmp, XMPConst.NS_SWF, "bgalpha", alpha);

		//Update forward lock
		setProperty(xmp, XMPConst.NS_SWF, "forwardlock", lock ? "True" : null);

		//Update the file if accepts XMP data.
		if (xmpFile.canPutXMP(xmp)) 
		{
			xmpFile.putXMP(xmp);
			task.statusDialog.text = localize("$$$/Metadata/Updated=The file was successfully updated.");
		}
		else
		{
			task.statusDialog.text =  localize("$$$/Metadata/Error=XMP data could not be saved.");
		}

		//Safely close the xmp file
		xmpFile.closeFile(XMPConst.CLOSE_UPDATE_SAFELY);
	}
	else
	{
		task.statusDialog.text =  localize("$$$/Metadata/NoFile=The file selected isn't either readable or not a SWF.");
	}

	//Once the task has been processed enable the Done button and disable Cancel
	task.statusDialog.enableDoneButton(true);
	task.statusDialog.enableCancelButton(false);
}