﻿/**
///////////////////////////////////////////////////////////////////////////////
ADOBE SYSTEMS INCORPORATED
Copyright 2008 Adobe Systems Incorporated
All Rights Reserved.
NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
terms of the Adobe license agreement accompanying it.  If you have received this file from a 
source other than Adobe, then your use, modification, or distribution of it requires the prior 
written permission of Adobe.
///////////////////////////////////////////////////////////////////////////////

This class is a wrapper for the URLStream class that provides utility methods to perform
HTTP specific requests. This file is intended as sample code and is not feature complete. 
The aim is demonstrating how developers can build their own libaries on top of the 
API provided by Adobe Device Central so it best fits the plug-ins requirements. 

The advantage of creating separete utility classes is that common functionality
can be re-used across tasks.
*/

//Class constructor

function HTTPClient(url, port)
{
	this.stream = new URLStream;
	this.stream.url = url;
	this.stream.port = port;
	this.multipart = false;
}

//Instance methods and properties

HTTPClient.prototype.authenticate = function(username, password) {
	this.stream.username = username;
	this.stream.password = password;
	this.stream.authentication = "basic"; 
}

HTTPClient.prototype.postForm = function(params) {
	if (params != undefined)
	{
		if (this.multipart)
		{
			//Create a "multipart/form-data" post that can send binary data
			var boundary = HTTPClient.createBoundary();
			this.stream.contentType = "multipart/form-data; boundary=" + boundary;		
			this.stream.postData = HTTPClient.buildMultipart(params, boundary);
		}
		else
		{
			//Create a default  "application/x-www-form-urlencoded" post
			var data = []
			for (var key in params) {data.push( key + "=" + params[key])};
			this.stream.postData = data.join("&");
		}
	}
	//If postData has been added it will perform a POST request otherwise a GET request
	this.stream.open();
}

HTTPClient.prototype.cancel = function() {
	this.stream.cancel();
}

HTTPClient.prototype.isConnected = function() {
	return this.stream.isConnected;
}

HTTPClient.prototype.result = function() {
	return this.stream.result;
}

HTTPClient.prototype.error = function() {
	return this.stream.error ;
}

HTTPClient.prototype.statusCode = function() {
	return this.stream.httpStatusCode;
}

HTTPClient.prototype.isResponseOK = function() {
	return (this.error() == 0 && this.statusCode() >= 200 && this.statusCode() < 300);
}

HTTPClient.prototype.isResponseApplicationError = function() {
	return (this.error() == 0 && this.statusCode().toString().match(/^4/)) ;
}

HTTPClient.prototype.errorDescription = function() {
	var description = HTTPClient.ERROR_CODES_DESCRIPTION[this.error()];			
	return localize(description || HTTPClient.ERROR_DESCRIPTION_DEFAULT, this.error());
}

HTTPClient.prototype.statusDescription = function() {
	var statusFirstDigit = this.statusCode().toString().match(/^\d/);
	var description = HTTPClient.STATUS_CODES_DESCRIPTION[statusFirstDigit];		
	return localize(description || HTTPClient.ERROR_DESCRIPTION_DEFAULT, this.statusCode());
}

//Class methods and properties 

HTTPClient.CONTENT_TYPES = {
	bmp : 'image/bmp',
	jpeg : 'image/jpeg', 
	jpg : 'image/jpg', 
	gif : 'image/gif',
	png : 'image/png',
	pdf : 'application/pdf',
	txt : 'text/plain',
	swf : 'application/x-shockwave-flash',
	flv : 'video/x-flv'	
}

HTTPClient.ERROR_CODES_DESCRIPTION = {
	0 : "$$$/Web/UploadSuccess=Request successfully processed.",
	4 : "$$$/Web/UrlMalformed=Could not connect. Invalid URL.",
	5 : "$$$/Web/ServerNotFound=Could not connect. Server not found.",
	6 : "$$$/Web/AccessDenied=Could not connect. Web access denied.",
	11 : "$$$/Web/UploadSuccess=File successfully uploaded.",
	18 : "$$$/Web/LoginDenied=Could not connect. Invalid username or password.",
}

HTTPClient.STATUS_CODES_DESCRIPTION = {
	1 : "$$$/Web/HTTPInformational=Request received, continuing process.",
	2 : "$$$/Web/HTTPSuccess=Request successfully processed.",
	3 : "$$$/Web/HTTPRedirection=Request has been redirected.",
	4 : "$$$/Web/HTTPClientError=Request is invalid or contains invalid data.",
	5 : "$$$/Web/HTTPServerError=Application error.",
}

HTTPClient.ERROR_DESCRIPTION_DEFAULT = "$$$/Web/UndefinedError=Undefined error occurred. Code %1"; 
HTTPClient.CONTENT_TYPE_DEFAULT = 'application/octet-stream';
HTTPClient.CHUNK = 65535;

HTTPClient.createBoundary = function() {
	var todayInMilliseconds = new Date().valueOf();
	var randomBoundary = todayInMilliseconds.toString(16);
	for (var i = 0; i < 20; i++ )
		randomBoundary += ( 97 + Math.random() * 25 ).toString(16);
	return randomBoundary ;
}

HTTPClient.buildMultipart = function(params, boundary) {
	
	function textPart(key, value) {
		var part = "Content-Disposition: form-data; name=\"" + key + "\"\r\n";
		part += "\r\n" + value;
		return part;
	}

	function filePart(key, file) {
		var part = "Content-Disposition: form-data; name=\"" + key + "\"; filename=\"" + file.displayName +"\"\r\n";
		part += "Content-Type: " + fileContentType(file.displayName) + "\r\n";
		part += "Content-Transfer-Encoding: binary\r\n";
		part += "\r\n";	
		part += readFile(file);
		return part;
	}

	function fileContentType(fileName) {
		var fileNameParts = fileName.split('.');
		var fileExtension = fileNameParts.length > 1 ? fileNameParts.pop() : '';
		return HTTPClient.CONTENT_TYPES[fileExtension.toLowerCase()] || HTTPClient.CONTENT_TYPE_DEFAULT;
	}

	function readFile(file) {
		var fileData = "";	
		file.encoding = "binary";
		file.open("r");
		while(!file.eof) {fileData += file.read (HTTPClient.CHUNK);}
		file.close();
		return fileData;
	}
	
	var multipartMessage = "";
	
	for (var key in params) {
		var value = params[key];
		multipartMessage += "--" + boundary + "\r\n";
		multipartMessage += (value instanceof File) ? filePart(key, value) : textPart(key, value);
		multipartMessage += "\r\n";
	}

	multipartMessage += "--" + boundary + "--\r\n";

	return multipartMessage;
}


