﻿/**
///////////////////////////////////////////////////////////////////////////////
ADOBE SYSTEMS INCORPORATED
Copyright 2008 Adobe Systems Incorporated
All Rights Reserved.
NOTICE:  Adobe permits you to use, modify, and distribute this file in accordance with the 
terms of the Adobe license agreement accompanying it.  If you have received this file from a 
source other than Adobe, then your use, modification, or distribution of it requires the prior 
written permission of Adobe.
///////////////////////////////////////////////////////////////////////////////

This sample plug-in demonstrates how to create an export task for your project that will 
publish your assets to a Web Application.

It shows how to:
1. Upload assets through a HTTP connection using the URLStream object
2. Verify the response sent by the web service 
3. Parse XML responses to show validation or application errors 
4. Prompt the user for input once the task has started running

For detailed information about this sample see the Readme.pdf file.

*/

#include "HTTPClient.jsx"

var gCanceled = false;
var gHTTPClient = null;

var allFiles = null;
var url  = null;
var port = null;
var authenticate = null;
var username  = null;
var password  = null;

// ------------------------------------------
// Helper methods
// ------------------------------------------

/** Parses the XML response returned by the web service and extracts and displays any error messages present.*/
function displayApplicationErrors(response, fileName)
{
	var message = localize("$$$/Web/ApplicationValidationFile=File '%1' could not be uploaded successfully.\n\nReason:\n", fileName);
	var errors = [];
	var responseXML = new XML(response);
	if (responseXML.xpath("/errors") != undefined)
	{
		for (var i = 0; i < responseXML.children().length(); i++)
			errors.push(responseXML.error[i]);
	}
	message += errors.length > 0 ? errors.join("\n") : response;
	alert(message, localize("$$$/Web/ApplicationValidation=Application failed to upload the file."));
}

/** Verifies if the element passed is part of a given list of items */
function inArray(list, elem)
{
	for(var i = 0; (i < list.length) ; i++)
		if(elem.path == list[i].path) return true;
	
	return false;
}

/** Checks if the value passed contains information */
function isBlank(value)
{
	return value == undefined || value.length <= 0
}

/** Builds an array of ListItems necessary to populate the fileList in the configuration dialog*/
function getSelectedResources() 
{
	var selectedFiles = project.selectedResources;
	var resources =[];
	for(var i = 0; i < selectedFiles.length; i++)
	{
		var file = new File(selectedFiles[i]);
		
		var item = new ListItem;
		item.filename = file.displayName;
		item.path = file.fsName;
		resources.push(item);
	}
	return resources;
}

/** Verifies that the input entered by the user meets the following requirements:
	1. Username and password must be entered if the authentication flag is set
	2. The URL and the port to connect must be set
	3. There must be at least 1 resource selected 
*/
function isValidInput() 
{
	errors = [];
	
	if (authenticate)
	{
		if(isBlank(username))
		{
			//Give the user another chance to input the username
			username = task.promptDialog(
				localize("$$$/Web/EnterUsername/Title=Empty Username"),
				localize("$$$/Web/EnterUsername/Label=Please enter a username:"),
				false);
				
			/*If the password is still blank record a validation error otherwise persist it in the configuration
			dialog so the next time the task is run it can be pre-populated */
			if(isBlank(username) ) 
				errors.push(localize("$$$/Web/NoUsernameDefined=No username defined."));
			else
				task.getTextField("username").text = username;
		}
		
		if(isBlank(password))
		{
			//Give the user another chance to input the password
			password = task.promptDialog(
				localize("$$$/Web/EnterPassword/Title=Empty Password"),
				localize("$$$/Web/EnterPassword/Label=Please enter a password:"),
				true);
			
			/*If the password is still blank record a validation error otherwise persist it in the configuration
			dialog so the next time the task is run it can be pre-populated */
			if(isBlank(password) )
				errors.push(localize("$$$/Web/NoPasswordDefined=No password defined."));
			else
				task.getPasswordField("password").text = password;
		}		
	}
	
	if(isBlank(allFiles))
		errors.push(localize("$$$/Web/NoSourceFiles=No source files selected."));
	
	if(isBlank(url))
		errors.push(localize("$$$/Web/NoURLDefined=No URL defined."));
		
	if(isBlank(port))
		errors.push(localize("$$$/Web/NoPortDefined=No port defined."));

	if (errors.length > 0) 
	{
		task.statusDialog.text = localize("$$$/Web/Canceled=Sending canceled:\n%1", errors.join(" "));
		return false;
	}
	
	return true;
}

/** Sets up and launches the status dialog */
function launchStatusDialog()
{
	var dialog = task.statusDialog;
	dialog.start("doCancel", "doDone");
	dialog.text = localize("$$$/Web/SendTo=Send to Web Server");
	dialog.title = localize("$$$/Web/ProcessingRequest=Processing request...");
	dialog.enableDoneButton(false);
	dialog.enableCancelButton(true);
}

/** If authenticate checkbox has been ticked enable username and password. 
Otherwise, disable them.*/
function toggleAuthentication()
{
	var authenticate = task.getCheckBox("authenticate").checked;
	task.getTextField("username").enabled = authenticate;
	task.getPasswordField("password").enabled = authenticate;
}

function toArray(o)
{
	var a = [];
	for(i = 0; i < o.length; i++)
		a.push(o[i]);
	
	return a;
}

// ------------------------------------------
// Methods invoked from the UI
// ------------------------------------------

/** Adds the files selected in the resources dialog to the fileList control in the 
configuration dialog. */
function doAddFile()
{
	var newFiles = project.openResourcesDialog ();
	
	if(newFiles != undefined)
	{
		var files = toArray(task.getFileList("files").list);
		
		for(var i = 0; i < newFiles.length; i++)
		{
			var f = new File(newFiles[i]);
			var item = new ListItem;
			item.filename = f.displayName;
			item.path = f.fsName;
			
			if(! inArray(files, item))
				files.push(item);
		}
		
		task.getFileList("files").list = files;
		
		if(files.length > 0)
			task.getButton("btnRemove").enabled = true;
	}
}

/** If the request has been canceled makes sure the connection is terminated and the status dialog removed */
function doCancel()
{
	gCanceled = true;
	gHTTPClient.cancel();
	task.statusDialog.end();
}

/** When the task is completed closes the status dialog and updates the global canceled variable */
function doDone()
{
	gCanceled = true;
	task.statusDialog.end();
}

/** Removes the files selected from the  fileList in the configuration dialog */
function doRemoveFile()
{
	var filesControl = task.getFileList("files");
	var selectedFiles = filesControl.selectedItems;
	var all = filesControl.list;
	var tmpFiles = [];
	for(i = 0; i < all.length; i++)
	{
			var found = false;
			for(j = 0; (j < selectedFiles.length) && ! found; j++)
			{
				if(all[i].path == selectedFiles[j].path)
					found = true;
			}
			if(! found)
				tmpFiles.push(all[i]);
	}
	if(tmpFiles.length <= 0)
		task.getButton("btnRemove").enabled = false;
		
	filesControl.list = tmpFiles;
}

// ------------------------------------------
// install
// ------------------------------------------

function install()
{
	/* When installing the task there's no files added to the files
	list. We won't be able to remove any so we need to disable
	the "btnRemove" */	
	task.getButton("btnRemove").enabled = false;
}

// ------------------------------------------
// configure
// ------------------------------------------

function configure(toolbar)
{
	if(toolbar)
	{
		/* If we have selected to run the task from the toolbar we need to add the 
		pre selected files to the fileList in the configuration dialog. */
		task.getFileList("files").list = getSelectedResources();
		
		/* The remove button should now be enabled since there's files that can 
		be removed from the list. */
		task.getButton("btnRemove").enabled = true;
	}

	//Make sure the authentication elements are properly enabled/disabled
	toggleAuthentication(); 
}

// ------------------------------------------
// execute
// ------------------------------------------

function execute()
{
	launchStatusDialog();
	
	//Retrieve all the input from the configuration dialog 
	allFiles = toArray(task.getElement("files").list);
	url  = task.getElement("url").text;
	port = task.getElement("port").text;
	authenticate = task.getElement("authenticate").checked;
	username  = task.getElement("username").text;
	password  = task.getElement("password").text; 
		
	//Validate the user input 
	task.statusDialog.text = localize("$$$/Web/ValidatingInput=Validating request details...");
	var error = ! isValidInput();

	var errorHeader = "$$$/Web/OperationStopped=The operation stopped with \"%1\".\n%2\nFiles following in the queue will not be transmitted.";
	var message = "";
	
	//Go through the files added and send one at a time. 
	for(var i = 0; (i < allFiles.length) && ! gCanceled && ! error; i++)
	{
		var file = new File(allFiles[i].path); 
		
		if(file.exists)
		{		
			task.statusDialog.text = localize("$$$/Web/Uploading=Processing file %1 of %2\nUploading \"%3\"...", i +1 , allFiles.length, file.displayName);
		
			gHTTPClient = new HTTPClient(url, port);
			if (authenticate)
				gHTTPClient.authenticate(username, password);
				
			//Set the request to multipart and post the data since we are sending binary
			gHTTPClient.multipart = true;
			gHTTPClient.postForm({"asset[uploaded_data]" : file});

			//Wait while we are making the request
			while(! gCanceled && gHTTPClient.isConnected()) { task.statusDialog.modalDuring(); }

			/* Once the request has been completed, set the correct message depending on the error or 
			http status code returned */
			message = gHTTPClient.error() ?  gHTTPClient.errorDescription() : gHTTPClient.statusDescription();
			error = !gHTTPClient.isResponseOK();
		}
		else
		{
			message = localize("$$$/Web/FileNotExisting=Source file doesn't exist.");
			error = true;
		}
	
		//Display any possible error messages returned from the application
		if (gHTTPClient.isResponseApplicationError())
			displayApplicationErrors(gHTTPClient.result(), file.displayName);
		
		task.statusDialog.text = (error ? localize(errorHeader, file.displayName, message) : message);
	}
		
	//Once the task has been processed enable the Done button and disable Cancel
	task.statusDialog.enableDoneButton(true);
	task.statusDialog.enableCancelButton(false);
	
	while(! gCanceled) { task.statusDialog.modalDuring(); }
}





